/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * ============================================
 *
 * Copyright (C) 2007-2010 by Tidalwave s.a.s.
 * Project home page: http://openbluesky.kenai.com
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * $Id$
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.role.util;

import javax.annotation.Nonnull;
import javax.annotation.PostConstruct;
import java.lang.reflect.Method;
import it.tidalwave.util.logging.Logger;

/***********************************************************************************************************************
 *
 * This class represents the abstract capability of loading data somewhere.
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public final class PostConstructorCaller
  {
    private final static String CLASS = PostConstructorCaller.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    public static void callPostConstructors (final @Nonnull Object object)
      {
        callPostConstructors(object, object.getClass());
      }

    /*******************************************************************************************************************
     *
     *
     ******************************************************************************************************************/
    private static void callPostConstructors (final @Nonnull Object object,
                                              final @Nonnull Class<?> clazz)
      {
        final Class<?> superclass = clazz.getSuperclass();

        if (superclass != null)
          {
            callPostConstructors(object, superclass);
          }

        for (final Method method : clazz.getDeclaredMethods())
          {
            if (method.getAnnotation(PostConstruct.class) != null)
              {
                try
                  {
                    method.setAccessible(true);
                    method.invoke(object);
                  }
                catch (Exception e)
                  {
                    logger.severe("While initializing %s: %s", object, e);
                    logger.throwing(CLASS, "callPostConstructors()", e);
                    e.printStackTrace();
                  }
              }
          }
      }
  }
